﻿using ELTE.BoardGame.Persistence;
using System;

namespace ELTE.BoardGame.Model
{
    /// <summary>
    /// Connect four játék modell típusa.
    /// </summary>
    public class ConnectFourModel : BoardGameModel
    {
        #region Private constants

        /// <summary>
        /// A nyeréshez szükséges mezők száma.
        /// </summary>
        private const Int32 NumberOfFields = 4;

        #endregion

        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe játék modell példányosítása.
        /// </summary>í
        /// <param name="boardSize">Játéktábla vízszintes, illetve függőleges mérete.</param>
        public ConnectFourModel(Int32 boardSize) 
            : base(boardSize)
        {
        }

        #endregion
        
        #region Protected methods

        /// <summary>
        /// Játék léptetése.
        /// </summary>
        /// <param name="x">Oszlop index.</param>
        /// <param name="y">Sor index.</param>
        protected override void StepGameInternal(Int32 x, Int32 y)
        {
            if (this.gameBoard[x, 0] != Player.NoPlayer) // ellenőrizzük az oszlopot
                throw new InvalidOperationException("Column is full!");

            Int32 i = BoardHeight - 1;
            while (this.gameBoard[x, i] != Player.NoPlayer)
                i--;

            this.gameBoard[x, i] = CurrentPlayer; // pozíció rögzítése
            this.OnFieldChanged(x, i, CurrentPlayer); // jelezzük egy eseménykiváltással, hogy változott a mező
        }

        /// <summary>
        /// Játék ellenőrzése.
        /// </summary>
        protected override void CheckGame() 
        {
            Player won = Player.NoPlayer;

            for (Int32 x = 0; x < BoardWidth && won == Player.NoPlayer; x++)
                for (Int32 y = 0; y < BoardHeight && won == Player.NoPlayer; y++)
                {
                    if (this.gameBoard[x, y] == Player.NoPlayer)
                        continue;

                    // ellenőrizzük, valamelyik irányban megvan-e a 4 jel
                     if (this.HaveSameOwner(x, y, 0, 1) || this.HaveSameOwner(x, y, 1, 0) || this.HaveSameOwner(x, y, 1, 1) || this.HaveSameOwner(x, y, -1, -1))
                        this.OnGameWon(this.gameBoard[x, y]); // esemény kiváltása
                }

            if (StepNumber == BoardHeight * BoardWidth) // döntetlen játék
            {
                this.OnGameOver(); // esemény kiváltása
            }
        }

        /// <summary>
        /// Mezősorozat azonos tulajdonosának ellenőrzése.
        /// </summary>
        /// <param name="startX">Kezdő oszlop.</param>
        /// <param name="startY">Kezdő sor.</param>
        /// <param name="deltaX">Oszlopbeli váltás.</param>
        /// <param name="deltaY">Sorbeli váltás.</param>
        private Boolean HaveSameOwner(Int32 startX, Int32 startY, Int32 deltaX, Int32 deltaY)
        {
            for (Int32 i = 1; i < 4; i++)
            {
                if (startX + deltaX * i < 0 || startX + deltaX * i >= BoardHeight ||
                    startY + deltaY * i < 0 || startY + deltaY * i >= BoardWidth)
                    return false;

                if (this.gameBoard[startX, startY] != this.gameBoard[startX + deltaX * i, startY + deltaY * i])
                    return false;
            }

            return true;
        }

        #endregion
    }
}
