﻿using System;
using ELTE.BoardGame.Persistence;

namespace ELTE.BoardGame.Model
{
    /// <summary>
    /// Tic-Tac-Toe játék modell típusa.
    /// </summary>
    public class TicTacToeModel : BoardGameModel
    {
        #region Private constants

        /// <summary>
        /// Játéktábla vízszintes, illetve függőleges mérete.
        /// </summary>
        private const Int32 BoardSize = 3;

        #endregion

        #region Constructors

        /// <summary>
        /// Tic-Tac-Toe játék modell példányosítása.
        /// </summary>
        public TicTacToeModel() : base(BoardSize)
        {

        }

        #endregion

        #region Protected methods

        /// <summary>
        /// Játék léptetése.
        /// </summary>
        /// <param name="x">Oszlop index.</param>
        /// <param name="y">Sor index.</param>
        protected override void StepGameInternal(Int32 x, Int32 y)
        {
            this.gameBoard[x, y] = this.CurrentPlayer; // pozíció rögzítése
            this.OnFieldChanged(x, y, this.CurrentPlayer); // jelezzük egy eseménykiváltással, hogy változott a mező
        }

        /// <summary>
        /// Játék ellenőrzése.
        /// </summary>
        protected override void CheckGame()
        {
            Player won = Player.NoPlayer;

            for (int i = 0; i < this.BoardWidth; ++i) // ellenőrzések végrehajtása
            {
                if (this.gameBoard[i, 0] != 0 && this.gameBoard[i, 0] == this.gameBoard[i, 1] && this.gameBoard[i, 1] == this.gameBoard[i, 2])
                    won = this.gameBoard[i, 0];
            }
            for (int i = 0; i < this.BoardHeight; ++i)
            {
                if (this.gameBoard[0, i] != 0 && this.gameBoard[0, i] == this.gameBoard[1, i] && this.gameBoard[1, i] == this.gameBoard[2, i])
                    won = this.gameBoard[0, i];
            }
            if (this.gameBoard[0, 0] != 0 && this.gameBoard[0, 0] == this.gameBoard[1, 1] && this.gameBoard[1, 1] == this.gameBoard[2, 2])
                won = this.gameBoard[0, 0];
            if (this.gameBoard[0, 2] != 0 && this.gameBoard[0, 2] == this.gameBoard[1, 1] && this.gameBoard[1, 1] == this.gameBoard[2, 0])
                won = this.gameBoard[0, 2];

            if (won != Player.NoPlayer) // ha valaki győzött
            {
                this.OnGameWon(won); // esemény kiváltása
            }
            else if (this.StepNumber == this.BoardWidth * this.BoardHeight) // döntetlen játék
            {
                this.OnGameOver(); // esemény kiváltása
            }
        }

        #endregion
    }
}
